from typing import (
    Dict,
)

from edpconfigprovider import ConfigProvider


class MultiConfigProvider(ConfigProvider):
    def __init__(self, providers: Dict[str, ConfigProvider]):
        self.providers = providers

    def get_configuration(self) -> Dict:
        ret = {}
        for key, provider in self.providers.items():
            ret[key] = provider.get_configuration()

        return ret
