from abc import (
    ABC,
    abstractmethod,
)

from edpconfigprovider import ConfigProvider


class ProviderBuilder(ABC):
    @abstractmethod
    def build_provider(self, **kwargs) -> ConfigProvider:
        raise NotImplementedError()
