from ._ConfigProviderException import ConfigProviderException


class NoConfigurationException(ConfigProviderException):
    def __init__(self, configuration_name: str, *args: object) -> None:
        super().__init__(f"No configuration for {configuration_name}", *args)
